(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 02-Feb-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: PixelMotionBlur.cpp
//
// Desc: This sample shows how to do a single pass motion blur effect using
//       floating point textures and multiple render targets.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit PixelMotionBlurUnit;

interface

uses
  Windows, Messages,
  Direct3D9, D3DX9, dxerr9,
  D3DApp, D3DFont, D3DUtil, DXUtil,
  Resource;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

{.$DEFINE DEBUG_VS}   // Uncomment this line to debug vertex shaders
{.$DEFINE DEBUG_PS}   // Uncomment this line to debug pixel shaders


//-----------------------------------------------------------------------------
// Globals variables and definitions
//-----------------------------------------------------------------------------
const
  NUM_OBJECTS = 40;
  NUM_WALLS = 250;
  MOVESTYLE_DEFAULT = 0;

type
  TScreenVertex = packed record
    pos: TD3DXVector4;
    clr: DWORD;
    tex1: TD3DXVector2;
  end;

const
  TScreenVertex_FVF = D3DFVF_XYZRHW or D3DFVF_DIFFUSE or D3DFVF_TEX1;

type
  PObjectStruct = ^TObjectStruct;
  TObjectStruct = record
  // public
    m_vWorldPos: TD3DXVector3;
    m_mWorld: TD3DXMatrixA16;
    m_mWorldLast: TD3DXMatrixA16;
    m_pMesh: ID3DXMesh;
    m_pMeshTexture: IDirect3DTexture9;
  end;

type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Main class to run this application. Most functionality is inherited
  //       from the CD3DApplication base class.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    m_pFont: ID3DXFont;              // Font for drawing text
    m_bShowHelp: Boolean;
    m_Camera: CFirstPersonCamera;
    m_pEffect: ID3DXEffect;

    m_Vertex: array[0..3] of TScreenVertex;

    m_pMesh1: ID3DXMesh;
    m_pMeshTexture1: IDirect3DTexture9;
    m_pMesh2: ID3DXMesh;
    m_pMeshTexture2: IDirect3DTexture9;
    m_pMeshTexture3: IDirect3DTexture9;

    m_pOriginalRenderTarget: IDirect3DSurface9;

    m_pFullScreenRenderTarget: IDirect3DTexture9;
    m_pFullScreenRenderTargetSurf: IDirect3DSurface9;

    m_pPixelVelocityTexture1: IDirect3DTexture9;
    m_pPixelVelocitySurf1: IDirect3DSurface9;
    m_pPixelVelocityTexture2: IDirect3DTexture9;
    m_pPixelVelocitySurf2: IDirect3DSurface9;

    m_pLastFrameVelocityTexture: IDirect3DTexture9;
    m_pLastFrameVelocitySurf:    IDirect3DSurface9;
    m_pCurFrameVelocityTexture: IDirect3DTexture9;
    m_pCurFrameVelocitySurf:    IDirect3DSurface9;

    m_fChangeTime: Single;
    m_bShowBlurFactor: Boolean;
    m_bShowUnblurred: Boolean;
    m_dwBackgroundColor: DWORD;

    m_fPixelBlurConst: Single;
    m_fObjectSpeed: Single;
    m_fCameraSpeed: Single;

    m_pScene1Object: array[0..NUM_OBJECTS-1] of PObjectStruct;
    m_pScene2Object: array[0..NUM_WALLS-1] of PObjectStruct;
    m_dwMoveSytle: DWORD; //todo: Need to fill bug report!!!
    m_nSleepTime: Integer;
    m_mViewProjectionLast: TD3DXMatrix;
    m_nCurrentScene: Integer;

    m_hWorld:                    TD3DXHandle;
    m_hWorldLast:                TD3DXHandle;
    m_hMeshTexture:              TD3DXHandle;
    m_hWorldViewProjection:      TD3DXHandle;
    m_hWorldViewProjectionLast:  TD3DXHandle;
    m_hCurFrameVelocityTexture:  TD3DXHandle;
    m_hLastFrameVelocityTexture: TD3DXHandle;
    m_hTechWorldWithVelocity:    TD3DXHandle;
    m_hPostProcessMotionBlur:    TD3DXHandle;

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;
    function LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
    procedure SetupFullscreenQuad;
    procedure RenderText;

  public
    constructor Create;
    destructor Destroy; override;
    function MsgProc(hWnd: HWND; uMsg: LongWord; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;



implementation

uses SysUtils;

const
  UnitName = 'BasicHLSL.pas';

{ CMyD3DApplication }

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.  
//       The application window has not yet been created and no Direct3D device 
//       has been created, so any initialization that depends on a window or 
//       Direct3D should be deferred to a later stage. 
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited Create;
  m_Camera:= CFirstPersonCamera.Create;

  m_strWindowTitle := 'Motion Blur';
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_dwCreationWidth  := 640;
  m_dwCreationHeight := 480;
  m_pFont := nil;

  m_pFullScreenRenderTarget := nil;
  m_pFullScreenRenderTargetSurf := nil;
  m_pPixelVelocityTexture1 := nil;
  m_pPixelVelocitySurf1 := nil;
  m_pPixelVelocityTexture2 := nil;
  m_pPixelVelocitySurf2 := nil;
  m_pLastFrameVelocityTexture := nil;
  m_pCurFrameVelocityTexture := nil;
  m_pCurFrameVelocitySurf := nil;
  m_pLastFrameVelocitySurf := nil;
  m_pEffect := nil;
  m_pOriginalRenderTarget := nil;
  m_nSleepTime := 0;
  m_fPixelBlurConst := 1.0;
  m_fObjectSpeed := 8.0;
  m_fCameraSpeed := 10.0;
  m_nCurrentScene := 1;

  m_fChangeTime := 0.0;
  m_dwMoveSytle := MOVESTYLE_DEFAULT;
  D3DXMatrixIdentity(m_mViewProjectionLast);

  m_hWorld := nil;
  m_hWorldLast := nil;
  m_hMeshTexture := nil;
  m_hWorldViewProjection := nil;
  m_hWorldViewProjectionLast := nil;
  m_hCurFrameVelocityTexture := nil;
  m_hLastFrameVelocityTexture := nil;
  m_hTechWorldWithVelocity := nil;
  m_hPostProcessMotionBlur := nil;

  m_pMesh1 := nil;
  m_pMeshTexture1 := nil;
  m_pMesh2 := nil;
  m_pMeshTexture2 := nil;
  m_pMeshTexture3 := nil;
  m_bShowBlurFactor := FALSE;
  m_bShowUnblurred := FALSE;
  m_bShowHelp := TRUE;
  m_dwBackgroundColor := $00003F3F;
end;

destructor CMyD3DApplication.Destroy;
begin
  FreeAndNil(m_Camera);
  inherited;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
var
  iObject: Integer;
begin
  try
    for iObject:= 0 to NUM_OBJECTS - 1 do
    begin
      GetMem(m_pScene1Object[iObject], SizeOf(TObjectStruct));
      ZeroMemory(m_pScene1Object[iObject], SizeOf(TObjectStruct));
    end;

    for iObject:= 0 to NUM_WALLS - 1 do
    begin
      GetMem(m_pScene2Object[iObject], SizeOf(TObjectStruct));
      ZeroMemory(m_pScene2Object[iObject], SizeOf(TObjectStruct));
    end;
  except
    Result:= E_OUTOFMEMORY;
    Exit;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  hDC: Windows.HDC;
  nHeight: Integer;
  dwShaderFlags: DWORD;
  str: array[0..MAX_PATH-1] of Char;
  fAspectRatio: Single;
  vecEye: TD3DXVector3;
  vecAt: TD3DXVector3;
  iObject: Integer;
  vPos: TD3DXVector3;
  mRot: TD3DXMatrix;
  mPos: TD3DXMatrix;
  mScale: TD3DXMatrix;
begin
  // Initialize the font
  hDC := GetDC(0);
  nHeight := -MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  ReleaseDC(0, hDC);
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, False, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'Arial', m_pFont);
  if FAILED(Result) then Exit;

  Result := LoadMesh('sphere.x', m_pMesh1);
    if FAILED(Result) then Exit;
  Result := LoadMesh('quad.x', m_pMesh2);
    if FAILED(Result) then Exit;

  // Create the mesh texture from a file
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'earth.bmp');
  Result := D3DXCreateTextureFromFile(m_pd3dDevice, str, m_pMeshTexture1);
    if FAILED(Result) then m_pMeshTexture1 := nil;
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'env2.bmp');
  Result := D3DXCreateTextureFromFile(m_pd3dDevice, str, m_pMeshTexture2);
    if FAILED(Result) then m_pMeshTexture2 := nil;
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'seafloor.bmp');
  Result := D3DXCreateTextureFromFile(m_pd3dDevice, str, m_pMeshTexture3);
    if FAILED(Result) then m_pMeshTexture3 := nil;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  DXUtil_FindMediaFileCb(str, SizeOf(str), 'PixelMotionBlur.fx');
  Result := D3DXCreateEffectFromFile(m_pd3dDevice, str, nil, nil, dwShaderFlags, nil, m_pEffect, nil);

  // If this fails, there should be debug output as to
  // they the .fx file failed to compile
  if FAILED(Result) then Exit;

  for iObject:= 0 to NUM_OBJECTS - 1 do
  begin
    m_pScene1Object[iObject].m_pMesh := m_pMesh1;
    m_pScene1Object[iObject].m_pMeshTexture := m_pMeshTexture1;
  end;

  for iObject:= 0 to NUM_WALLS - 1 do
  begin
    m_pScene2Object[iObject].m_pMesh := m_pMesh2;

    if (iObject < NUM_WALLS/5*1) then
    begin
      m_pScene2Object[iObject].m_pMeshTexture := m_pMeshTexture3;

      // Center floor
      vPos.x := 0.0;
      vPos.y := 0.0;
      vPos.z := (iObject-NUM_WALLS/5*0) * 1.0 + 10.0;

      D3DXMatrixRotationX(mRot, -D3DX_PI/2.0);
      D3DXMatrixScaling(mScale, 1.0, 1.0, 1.0);
    end
    else if (iObject < NUM_WALLS/5*2) then
    begin
      m_pScene2Object[iObject].m_pMeshTexture := m_pMeshTexture3;

      // Right floor
      vPos.x := 1.0;
      vPos.y := 0.0;
      vPos.z := (iObject-NUM_WALLS/5*1) * 1.0 + 10.0;

      D3DXMatrixRotationX(mRot, -D3DX_PI/2.0);
      D3DXMatrixScaling(mScale, 1.0, 1.0, 1.0);
    end
    else if (iObject < NUM_WALLS/5*3) then
    begin
      m_pScene2Object[iObject].m_pMeshTexture := m_pMeshTexture3;

      // Left floor
      vPos.x := -1.0;
      vPos.y := 0.0;
      vPos.z := (iObject-NUM_WALLS/5*2) * 1.0 + 10.0;

      D3DXMatrixRotationX(mRot, -D3DX_PI/2.0);
      D3DXMatrixScaling(mScale, 1.0, 1.0, 1.0);
    end
    else if (iObject < NUM_WALLS/5*4) then
    begin
      m_pScene2Object[iObject].m_pMeshTexture := m_pMeshTexture2;

      // Right wall
      vPos.x := 1.5;
      vPos.y := 0.5;
      vPos.z := (iObject-NUM_WALLS/5*3) * 1.0 + 10.0;

      D3DXMatrixRotationY(mRot, -D3DX_PI/2.0);
      D3DXMatrixScaling(mScale, 1.0, 1.0, 1.0);
    end
    else if (iObject < NUM_WALLS/5*5) then 
    begin
      m_pScene2Object[iObject].m_pMeshTexture := m_pMeshTexture2;

      // Left wall
      vPos.x := -1.5;
      vPos.y := 0.5;
      vPos.z := (iObject-NUM_WALLS/5*4) * 1.0 + 10.0;

      D3DXMatrixRotationY(mRot, D3DX_PI/2.0);
      D3DXMatrixScaling(mScale, 1.0, 1.0, 1.0);
    end;

    // Update the current world matrix for this object
    D3DXMatrixTranslation(mPos, vPos.x, vPos.y, vPos.z);
    D3DXMatrixMultiply(m_pScene2Object[iObject].m_mWorld, mRot, mPos);
    D3DXMatrixMultiply(m_pScene2Object[iObject].m_mWorld, mScale, m_pScene2Object[iObject].m_mWorld);

    // The walls don't move so just copy the current world matrix
    m_pScene2Object[iObject].m_mWorldLast := m_pScene2Object[iObject].m_mWorld;
  end;

  // Setup the camera
  fAspectRatio := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  m_Camera.SetProjParams(D3DX_PI/4, fAspectRatio, 1.0, 1000.0);

  vecEye:= D3DXVector3(40.0, 0.0, -15.0);
  vecAt := D3DXVector3( 4.0, 4.0, -15.0);
  m_Camera.SetViewParams(vecEye, vecAt);

  m_Camera.SetScalers(0.01, m_fCameraSpeed);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: LoadMesh()
// Desc: Load the mesh and ensure the mesh has normals, and optimize
//       the mesh for this graphics card's vertex cache so the triangle list
//       inside in the mesh will cache miss less which improves perf.
//-----------------------------------------------------------------------------
function CMyD3DApplication.LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
var
  pMesh: ID3DXMesh;
  str: array[0..MAX_PATH-1] of Char;
  rgdwAdjacency: PDWORD;
  pTempMesh: ID3DXMesh;
begin
  if (@ppMesh = nil) then
  begin
    Result:= E_INVALIDARG;
    Exit;
  end;

  // Load the mesh with D3DX and get back a ID3DXMesh*.  For this
  // sample we'll ignore the X file's embedded materials since we know
  // exactly the model we're loading.  See the mesh samples such as
  // "OptimizedMesh" for a more generic mesh loading example.
  Result := DXUtil_FindMediaFileCb(str, SizeOf(str), strFileName);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', D3DAPPERR_MEDIANOTFOUND, UnitName);
    Exit;
  end;
  Result := D3DXLoadMeshFromX(str, D3DXMESH_MANAGED,
                              m_pd3dDevice, nil, nil, nil, nil, pMesh);
  if FAILED(Result) or (pMesh = nil) then
  begin
    Result:= DXTRACE_ERR('D3DXLoadMeshFromX', Result, UnitName);
    Exit;
  end;

  // Make sure there are normals which are required for lighting
  if (pMesh.GetFVF and D3DFVF_NORMAL = 0) then
  begin
    Result := pMesh.CloneMeshFVF(pMesh.GetOptions,
                                 pMesh.GetFVF or D3DFVF_NORMAL,
                                 m_pd3dDevice, pTempMesh);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('CloneMeshFVF', Result, UnitName);
      Exit;
    end;

    D3DXComputeNormals(pTempMesh, nil);

    SAFE_RELEASE(pMesh);
    pMesh := pTempMesh;
  end;

  // Optimize the mesh for this graphics card's vertex cache
  // so when rendering the mesh's triangle list the vertices will
  // cache hit more often so it won't have to re-execute the vertex shader
  // on those vertices so it will improves perf.
  try
    GetMem(rgdwAdjacency, SizeOf(DWORD) * pMesh.GetNumFaces * 3);
  except
    Result:= E_OUTOFMEMORY;
    Exit;
  end;
  pMesh.ConvertPointRepsToAdjacency(nil, rgdwAdjacency);
  pMesh.OptimizeInplace(D3DXMESHOPT_VERTEXCACHE, rgdwAdjacency, nil, nil, nil);
  FreeMem(rgdwAdjacency);

  ppMesh := pMesh;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  colorWhite: TD3DXColor;
  colorBlack: TD3DXColor;
  colorAmbient: TD3DXColor;
  colorSpecular: TD3DXColor;
  desc: TD3DSurfaceDesc;
  fVelocityCapInPixels: Single;
  fVelocityCapNonHomogeneous: Single;
  fVelocityCapSqNonHomogeneous: Single;
begin
  if Assigned(m_pFont) then m_pFont.OnResetDevice;
  if Assigned(m_pEffect) then m_pEffect.OnResetDevice;

  m_Camera.SetResetCursorAfterMove(not m_bWindowed);

  // Create a A8R8G8B8 render target texture.  This will be used to render
  // the full screen and then rendered to the backbuffer using a quad.
  Result := D3DXCreateTexture(m_pd3dDevice,
                              m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height,
                              1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8,
                              D3DPOOL_DEFAULT, m_pFullScreenRenderTarget);
  if FAILED(Result) then Exit;

  // Create two D3DFMT_G16R16F render targets.  These will be used to store
  // velocity of each pixel (one for the current frame, and one for last frame).
  Result := D3DXCreateTexture(m_pd3dDevice,
                              m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height,
                              1, D3DUSAGE_RENDERTARGET, D3DFMT_G16R16F,
                              D3DPOOL_DEFAULT, m_pPixelVelocityTexture1);
  if FAILED(Result) then Exit;
  Result := D3DXCreateTexture(m_pd3dDevice,
                              m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height,
                              1, D3DUSAGE_RENDERTARGET, D3DFMT_G16R16F,
                              D3DPOOL_DEFAULT, m_pPixelVelocityTexture2);
  if FAILED(Result) then Exit;

  // Store pointers to surfaces so we can call SetRenderTarget() later
  m_pPixelVelocityTexture1.GetSurfaceLevel(0, m_pPixelVelocitySurf1);
  m_pPixelVelocityTexture2.GetSurfaceLevel(0, m_pPixelVelocitySurf2);

  // Setup the current & last pointers that are swapped every frame.
  m_pCurFrameVelocityTexture := m_pPixelVelocityTexture1;
  m_pLastFrameVelocityTexture := m_pPixelVelocityTexture2;
  m_pCurFrameVelocitySurf := m_pPixelVelocitySurf1;
  m_pLastFrameVelocitySurf := m_pPixelVelocitySurf2;

  m_pFullScreenRenderTarget.GetSurfaceLevel(0, m_pFullScreenRenderTargetSurf);

  SetupFullscreenQuad;

  colorWhite:= D3DXColor(1.0, 1.0, 1.0, 1.0);
  colorBlack:= D3DXColor(0.0, 0.0, 0.0, 1.0);
  colorAmbient:= D3DXColor(0.35, 0.35, 0.35, 0);
  colorSpecular:= D3DXColor(0.5, 0.5, 0.5, 1.0);
  m_pEffect.SetVector('MaterialAmbientColor', PD3DXVector4(@colorAmbient)^);
  m_pEffect.SetVector('MaterialDiffuseColor', PD3DXVector4(@colorWhite)^);
  m_pEffect.SetTexture('RenderTargetTexture', m_pFullScreenRenderTarget);

  m_pFullScreenRenderTargetSurf.GetDesc(desc);
  m_pEffect.SetFloat('RenderTargetWidth', desc.Width);
  m_pEffect.SetFloat('RenderTargetHeight', desc.Height);

  // 12 is the number of samples in our post-process pass, so we don't want
  // pixel velocity of more than 12 pixels or else we'll see artifacts
  fVelocityCapInPixels := 3.0;
  fVelocityCapNonHomogeneous := fVelocityCapInPixels * 2 / m_d3dsdBackBuffer.Width;
  fVelocityCapSqNonHomogeneous := fVelocityCapNonHomogeneous * fVelocityCapNonHomogeneous;

  m_pEffect.SetFloat('VelocityCapSq', fVelocityCapSqNonHomogeneous);
  m_pEffect.SetFloat('ConvertToNonHomogeneous', 1.0 / m_d3dsdBackBuffer.Width);

  m_hTechWorldWithVelocity    := m_pEffect.GetTechniqueByName('WorldWithVelocity');
  m_hPostProcessMotionBlur    := m_pEffect.GetTechniqueByName('PostProcessMotionBlur');
  m_hWorld                    := m_pEffect.GetParameterByName(nil, 'mWorld');
  m_hWorldLast                := m_pEffect.GetParameterByName(nil, 'mWorldLast');
  m_hWorldViewProjection      := m_pEffect.GetParameterByName(nil, 'mWorldViewProjection');
  m_hWorldViewProjectionLast  := m_pEffect.GetParameterByName(nil, 'mWorldViewProjectionLast');
  m_hMeshTexture              := m_pEffect.GetParameterByName(nil, 'MeshTexture');
  m_hCurFrameVelocityTexture  := m_pEffect.GetParameterByName(nil, 'CurFrameVelocityTexture');
  m_hLastFrameVelocityTexture := m_pEffect.GetParameterByName(nil, 'LastFrameVelocityTexture');

  // Turn off lighting since its done in the shaders
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iFalse);

  // Save a pointer to the orignal render target to restore it later
  Result := m_pd3dDevice.GetRenderTarget(0, m_pOriginalRenderTarget);
  if FAILED(Result) then Exit;

  // Clear each RT
  m_pd3dDevice.SetRenderTarget(0, m_pFullScreenRenderTargetSurf);
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);
  m_pd3dDevice.SetRenderTarget(0, m_pLastFrameVelocitySurf);
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);
  m_pd3dDevice.SetRenderTarget(0, m_pCurFrameVelocitySurf);
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);

  // Restore the orignal RT
  m_pd3dDevice.SetRenderTarget(0, m_pOriginalRenderTarget);
  m_pd3dDevice.SetRenderTarget(1, nil);
end;


//-----------------------------------------------------------------------------
// Name: SetupFullscreenQuad()
// Desc: Sets up a full screen quad.  First we render to a fullscreen render
//       target texture, and then we render that texture using this quad to
//       apply a pixel shader on every pixel of the scene.
//-----------------------------------------------------------------------------
procedure CMyD3DApplication.SetupFullscreenQuad;
var
  desc: TD3DSurfaceDesc;
  fWidth5, fHeight5, fTexWidth1, fTexHeight1: Single;
begin
  m_pFullScreenRenderTargetSurf.GetDesc(desc);

  // Ensure that we're directly mapping texels to pixels by offset by 0.5
  // For more info see the doc page titled "Directly Mapping Texels to Pixels"
  fWidth5  := m_d3dsdBackBuffer.Width - 0.5;
  fHeight5 := m_d3dsdBackBuffer.Height - 0.5;

  fTexWidth1  := m_d3dsdBackBuffer.Width / desc.Width;
  fTexHeight1 := m_d3dsdBackBuffer.Height / desc.Height;

  // Fill in the vertex values
  m_Vertex[0].pos := D3DXVector4(fWidth5, -0.5, 0.0, 1.0);
  m_Vertex[0].clr := D3DXColorToDWord(D3DXColor(0.5, 0.5, 0.5, 0.66666));
  m_Vertex[0].tex1 := D3DXVector2(fTexWidth1, 0.0);

  m_Vertex[1].pos := D3DXVector4(fWidth5, fHeight5, 0.0, 1.0);
  m_Vertex[1].clr := D3DXColorToDWord(D3DXColor(0.5, 0.5, 0.5, 0.66666));
  m_Vertex[1].tex1 := D3DXVector2(fTexWidth1, fTexHeight1);

  m_Vertex[2].pos := D3DXVector4(-0.5, -0.5, 0.0, 1.0);
  m_Vertex[2].clr := D3DXColorToDWord(D3DXColor(0.5, 0.5, 0.5, 0.66666));
  m_Vertex[2].tex1 := D3DXVector2(0.0, 0.0);

  m_Vertex[3].pos := D3DXVector4(-0.5, fHeight5, 0.0, 1.0);
  m_Vertex[3].clr := D3DXColorToDWord(D3DXColor(0.5, 0.5, 0.5, 0.66666));
  m_Vertex[3].tex1 := D3DXVector2(0.0, fTexHeight1);
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
var
  pTempTex: IDirect3DTexture9;
  pTempSurf: IDirect3DSurface9;
  iObject: Integer;
  vPos: TD3DXVector3;
  fRadius: Single;
begin
  // Swap the current frame's per-pixel velocity texture with
  // last frame's per-pixel velocity texture
  pTempTex := m_pCurFrameVelocityTexture;
  m_pCurFrameVelocityTexture := m_pLastFrameVelocityTexture;
  m_pLastFrameVelocityTexture := pTempTex;

  pTempSurf := m_pCurFrameVelocitySurf;
  m_pCurFrameVelocitySurf := m_pLastFrameVelocitySurf;
  m_pLastFrameVelocitySurf := pTempSurf;

  m_pEffect.SetFloat('PixelBlurConst', m_fPixelBlurConst);

  // Update the camera's postion based on user input
  m_Camera.FrameMove(m_fElapsedTime);

  if (m_nCurrentScene = 1) then
  begin
    // Move the objects around based on some simple function
    for iObject:= 0 to NUM_OBJECTS - 1 do
    begin
      fRadius := 7.0;
      if (iObject >= 30) and (iObject < 41) then
      begin
        vPos.x := Cos(m_fObjectSpeed*0.125*m_fTime + 2*D3DX_PI/10*(iObject-30)) * fRadius;
        vPos.y := 10.0;
        vPos.z := Sin(m_fObjectSpeed*0.125*m_fTime + 2*D3DX_PI/10*(iObject-30)) * fRadius - 25.0;
      end
      else if (iObject >= 20) and (iObject < 31) then
      begin
        vPos.x := Cos(m_fObjectSpeed*0.25*m_fTime + 2*D3DX_PI/10*(iObject-20)) * fRadius;
        vPos.y := 10.0;
        vPos.z := Sin(m_fObjectSpeed*0.25*m_fTime + 2*D3DX_PI/10*(iObject-20)) * fRadius - 5.0;
      end
      else if (iObject >= 10) and (iObject < 21) then
      begin
        vPos.x := Cos(m_fObjectSpeed*0.5*m_fTime + 2*D3DX_PI/10*(iObject-10)) * fRadius;
        vPos.y := 0.0;
        vPos.z := Sin(m_fObjectSpeed*0.5*m_fTime + 2*D3DX_PI/10*(iObject-10)) * fRadius - 25.0;
      end
      else
      begin
        vPos.x := Cos(m_fObjectSpeed*m_fTime + 2*D3DX_PI/10*iObject) * fRadius;
        vPos.y := 0.0;
        vPos.z := Sin(m_fObjectSpeed*m_fTime + 2*D3DX_PI/10*iObject) * fRadius - 5.0;
      end;

      m_pScene1Object[iObject].m_vWorldPos := vPos;

      // Store the last world matrix so that we can tell the effect file
      // what it was when we render this object
      m_pScene1Object[iObject].m_mWorldLast := m_pScene1Object[iObject].m_mWorld;

      // Update the current world matrix for this object
      D3DXMatrixTranslation(m_pScene1Object[iObject].m_mWorld,
                            m_pScene1Object[iObject].m_vWorldPos.x,
                            m_pScene1Object[iObject].m_vWorldPos.y,
                            m_pScene1Object[iObject].m_vWorldPos.z);
    end;
  end;

  // Sleep to simulate time spent doing complex
  // graphics, AI, physics, networking, etc
  if (m_nSleepTime > 0) then Sleep(m_nSleepTime);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  pOriginalRenderTarget: IDirect3DSurface9;
  mProj: TD3DXMatrixA16;
  mView: TD3DXMatrixA16;
  mViewProjection: TD3DXMatrixA16;
  mWorldViewProjection: TD3DXMatrixA16;
  mWorldViewProjectionLast: TD3DXMatrixA16;
  iPass, cPasses: LongWord;
  iObject: Integer;
begin
  // Save a pointer to the current render target in the swap chain
  Result := m_pd3dDevice.GetRenderTarget(0, pOriginalRenderTarget);
  if FAILED(Result) then Exit;

  m_pEffect.SetTexture(m_hCurFrameVelocityTexture, m_pCurFrameVelocityTexture);
  m_pEffect.SetTexture(m_hLastFrameVelocityTexture, m_pLastFrameVelocityTexture);

  // Clear the velocity render target to 0
  m_pd3dDevice.SetRenderTarget(0, m_pCurFrameVelocitySurf);
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);

  // Clear the full screen render target to the background color
  m_pd3dDevice.SetRenderTarget(0, m_pFullScreenRenderTargetSurf);
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, m_dwBackgroundColor, 1.0, 0);

  // Set the render target 0 to be the fullscreen render target
  m_pd3dDevice.SetRenderTarget(0, m_pFullScreenRenderTargetSurf);

  // Set render target 1.  The pixel shader can then output to both
  // render targets in 1 pass if multiple render targets are supported
  m_pd3dDevice.SetRenderTarget(1, m_pCurFrameVelocitySurf);

  // Turn on Z for this pass
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iTrue);

  // For the first pass we'll draw the screen to the full screen render target
  // and to update the velocity render target with the velocity of each pixel
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Set world drawing technique
    m_pEffect.SetTechnique(m_hTechWorldWithVelocity);

    // Get the projection & view matrix from the camera class
    mProj := m_Camera.GetProjMatrix^;
    mView := m_Camera.GetViewMatrix^;

    // mViewProjection = mView * mProj;
    D3DXMatrixMultiply(mViewProjection, mView, mProj);

    if (m_nCurrentScene = 1) then
    begin
      // For each object, tell the effect about the object's current world matrix
      // and its last frame's world matrix and render the object.
      // The vertex shader can then use both these matricies to calculate how
      // much each vertex has moved.  The pixel shader then interpolates this
      // vertex velocity for each pixel
      for iObject:= 0 to NUM_OBJECTS - 1 do
      begin
        D3DXMatrixMultiply(mWorldViewProjection, m_pScene1Object[iObject].m_mWorld, mViewProjection);
        D3DXMatrixMultiply(mWorldViewProjectionLast, m_pScene1Object[iObject].m_mWorldLast, m_mViewProjectionLast);

        // Tell the effect where the camera is now
        m_pEffect.SetMatrix(m_hWorldViewProjection, mWorldViewProjection);
        m_pEffect.SetMatrix(m_hWorld, m_pScene1Object[iObject].m_mWorld);

        // Tell the effect where the camera was last frame
        m_pEffect.SetMatrix(m_hWorldViewProjectionLast, mWorldViewProjectionLast);

        // Tell the effect the current mesh's texture
        m_pEffect.SetTexture(m_hMeshTexture, m_pScene1Object[iObject].m_pMeshTexture);

        m_pEffect._Begin(@cPasses, 0);
        for iPass := 0 to cPasses - 1 do
        begin
          m_pEffect.Pass(iPass);
          m_pScene1Object[iObject].m_pMesh.DrawSubset(0);
        end;
        m_pEffect._End;
      end;
    end
    else if (m_nCurrentScene = 2) then
    begin
      for iObject:= 0 to NUM_WALLS - 1 do
      begin
        D3DXMatrixMultiply(mWorldViewProjection, m_pScene2Object[iObject].m_mWorld, mViewProjection);
        D3DXMatrixMultiply(mWorldViewProjectionLast, m_pScene2Object[iObject].m_mWorldLast, m_mViewProjectionLast);

        // Tell the effect where the camera is now
        m_pEffect.SetMatrix(m_hWorldViewProjection, mWorldViewProjection);
        m_pEffect.SetMatrix(m_hWorld, m_pScene2Object[iObject].m_mWorld);

        // Tell the effect where the camera was last frame
        m_pEffect.SetMatrix(m_hWorldViewProjectionLast, mWorldViewProjectionLast);

        // Tell the effect the current mesh's texture
        m_pEffect.SetTexture(m_hMeshTexture, m_pScene2Object[iObject].m_pMeshTexture);

        m_pEffect._Begin(@cPasses, 0);
        for iPass := 0 to cPasses - 1 do
        begin
          m_pEffect.Pass(iPass);
          m_pScene2Object[iObject].m_pMesh.DrawSubset(0);
        end;
        m_pEffect._End;
      end;
    end;

    m_pd3dDevice.EndScene;
  end;

  // Remember the current view projection matrix for next frame
  m_mViewProjectionLast := mViewProjection;

  // Now that we have the scene rendered into m_pFullScreenRenderTargetSurf
  // and the pixel velocity rendered into m_pCurFrameVelocitySurf
  // we can do a final pass to render this into the backbuffer and use
  // a pixel shader to blur the pixels based on the last frame's & current frame's
  // per pixel velocity to achieve a motion blur effect
  m_pd3dDevice.SetRenderTarget(0, pOriginalRenderTarget);
  m_pd3dDevice.SetRenderTarget(1, nil);
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iFalse);

  // Clear the render target
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);

  // Above we rendered to a fullscreen render target texture, and now we
  // render that texture using a quad to apply a pixel shader on every pixel of the scene.
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    m_pEffect.SetTechnique( m_hPostProcessMotionBlur);

    m_pEffect._Begin(@cPasses, 0);
    for iPass := 0 to cPasses - 1 do
    begin
      m_pEffect.Pass(iPass);
      m_pd3dDevice.SetFVF(TScreenVertex_FVF);
      m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, m_Vertex, SizeOf(TScreenVertex));
    end;
    m_pEffect._End;

    RenderText;

    m_pd3dDevice.EndScene;
  end;

  SAFE_RELEASE(pOriginalRenderTarget);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Draw the help & statistics for running sample
//-----------------------------------------------------------------------------
procedure CMyD3DApplication.RenderText;
var
  rc: TRect;
  iLineY: Integer;
  sz: array[0..99] of Char;
begin
  if (nil = m_pFont) then Exit;

{$IFDEF UNICODE}
  For UNICODE you should use "m_pFont.DrawTextW" routine!!!
{$ENDIF}

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  m_pFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15); 
  m_pFont.DrawTextA(nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));

  StrFmt(sz, 'Blur=%0.2f Object Speed=%0.1f Camera Speed=%0.1f', [m_fPixelBlurConst, m_fObjectSpeed, m_fCameraSpeed]);
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));

  if (m_nSleepTime = 0)
    then StrCopy(sz, 'Not sleeping between frames')
    else StrFmt(sz, 'Sleeping %dms per frame', [m_nSleepTime]);
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));

  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  m_pFont.DrawTextA(nil, 'Press ''End'' to toggle blur and ''P'' to change scene.',
                    -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));

  // Draw help
  if m_bShowHelp then
  begin
    iLineY := m_d3dsdBackBuffer.Height-15*6;
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    m_pFont.DrawTextA(nil, 'Controls:', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    SetRect(rc, 20, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Look: Left drag mouse'#10 +
                           'Move: A,W,S,D or Arrow Keys'#10 +
                           'Strafe up/down: Q,E or PgUp,PgDn'#10 +
                           'Change Scene: P'#10 +
                           'Framerate adjust: 0,1,2,3,4,5'#10,
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));

    SetRect(rc, 250, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Change Blur Factor: Insert,Delete,R'#10 +
                           'Turn Blur On/Off: Delete'#10 +
                           'Change Object Speed: Y,U,I'#10 +
                           'Change Camera Speed: H,J,K'#10 +
                           'Reset camera: Home'#10,
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));

    SetRect(rc, 500, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Hide help: F1'#10 +
                           'Change Device: F2'#10 +
                           'View readme: F9'#10 +
                           'Quit: ESC',
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));

  end else
  begin
    SetRect(rc, 2, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Press F1 for help',
                      -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
  end;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  if Assigned(m_pFont) then m_pFont.OnLostDevice;
  if Assigned(m_pEffect) then  m_pEffect.OnLostDevice;

  SAFE_RELEASE(m_pFullScreenRenderTargetSurf);
  SAFE_RELEASE(m_pFullScreenRenderTarget);
  SAFE_RELEASE(m_pPixelVelocitySurf1);
  SAFE_RELEASE(m_pPixelVelocityTexture1);
  SAFE_RELEASE(m_pPixelVelocitySurf2);
  SAFE_RELEASE(m_pPixelVelocityTexture2);
  SAFE_RELEASE(m_pOriginalRenderTarget);

  // Delphi special
  m_pLastFrameVelocityTexture:= nil;
  m_pLastFrameVelocitySurf:= nil;
  m_pCurFrameVelocityTexture:= nil;
  m_pCurFrameVelocitySurf:= nil;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
var
  iObject: Integer;
begin
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pEffect);

  SAFE_RELEASE(m_pFullScreenRenderTargetSurf);
  SAFE_RELEASE(m_pFullScreenRenderTarget);

  SAFE_RELEASE(m_pMesh1);
  SAFE_RELEASE(m_pMeshTexture1);
  SAFE_RELEASE(m_pMesh2);
  SAFE_RELEASE(m_pMeshTexture2);
  SAFE_RELEASE(m_pMeshTexture3);

  // Delphi special
  for iObject:= 0 to NUM_OBJECTS - 1 do
  begin
    m_pScene1Object[iObject].m_pMesh := nil;
    m_pScene1Object[iObject].m_pMeshTexture := nil;
  end;

  for iObject:= 0 to NUM_WALLS - 1 do
  begin
    m_pScene2Object[iObject].m_pMesh := nil;
    m_pScene2Object[iObject].m_pMeshTexture := nil;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
var
  iObject: Integer;
begin
  for iObject:= 0 to NUM_OBJECTS - 1 do
    FreeMem(m_pScene1Object[iObject]);

  for iObject:= 0 to NUM_WALLS - 1 do
    FreeMem(m_pScene2Object[iObject]);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
     ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) then Exit;
  {$ENDIF}

  if (dwBehavior and D3DCREATE_PUREDEVICE <> 0) then Exit;

  // No fallback, so need ps2.0
  if (pCaps.PixelShaderVersion < D3DPS_VERSION(2,0)) then Exit;

  // No fallback, so need multiple render targets
  if (pCaps.NumSimultaneousRTs < 2) then Exit;

  // If device doesn't support 1.1 vertex shaders in HW, switch to SWVP.
  if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,1)) then
  begin
    if ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  end;

  // No fallback, so need to support render target
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
              adapterFormat, D3DUSAGE_RENDERTARGET,
              D3DRTYPE_TEXTURE, backBufferFormat)) then Exit;

  // No fallback, so need to support D3DFMT_G16R16F render target
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
              adapterFormat, D3DUSAGE_RENDERTARGET,
              D3DRTYPE_TEXTURE, D3DFMT_G16R16F)) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
var
  s_fRemember: Single = 0.0;

function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: LongWord;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  vecEye: TD3DXVector3;
  vecAt: TD3DXVector3;
begin
  m_Camera.HandleMessages(hWnd, uMsg, wParam, lParam);

  case uMsg of
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDM_OPTIONS_FPS_SETTING_0: m_nSleepTime := 0;
        IDM_OPTIONS_FPS_SETTING_1: m_nSleepTime := 10;
        IDM_OPTIONS_FPS_SETTING_2: m_nSleepTime := 20;
        IDM_OPTIONS_FPS_SETTING_3: m_nSleepTime := 40;
        IDM_OPTIONS_FPS_SETTING_4: m_nSleepTime := 80;
        IDM_OPTIONS_FPS_SETTING_5: m_nSleepTime := 160;

        IDM_OPTIONS_INCREASEBLUR: m_fPixelBlurConst := m_fPixelBlurConst + 0.1;
        IDM_OPTIONS_DECREASEBLUR: m_fPixelBlurConst := m_fPixelBlurConst - 0.1;
        IDM_OPTIONS_RESETBLUR: m_fPixelBlurConst := 1.0;
        IDM_OPTIONS_TOGGLEBLUR:
        begin
          if (m_fPixelBlurConst = 0.0)
          then m_fPixelBlurConst := s_fRemember
          else
          begin
            s_fRemember := m_fPixelBlurConst;
            m_fPixelBlurConst := 0.0;
          end;
        end;

        IDM_OPTIONS_INCREASEOBJSPEED: m_fObjectSpeed := m_fObjectSpeed + 0.1;
        IDM_OPTIONS_DECREASEOBJSPEED: m_fObjectSpeed := m_fObjectSpeed - 0.1;
        IDM_OPTIONS_RESETOBJSPEED: m_fObjectSpeed := 8.0;

        IDM_OPTIONS_INCREASECAMSPEED:
        begin
          m_fCameraSpeed := m_fCameraSpeed + 1.0;
          m_Camera.SetScalers(0.01, m_fCameraSpeed);
        end;
        IDM_OPTIONS_DECREASECAMSPEED:
        begin
          m_fCameraSpeed := m_fCameraSpeed - 1.0;
          m_Camera.SetScalers(0.01, m_fCameraSpeed);
        end;
        IDM_OPTIONS_RESETCAMSPEED:
        begin
          m_fCameraSpeed := 25.0;
          m_Camera.SetScalers(0.01, m_fCameraSpeed);
        end;

        IDM_OPTIONS_CHANGESCENE:
        begin
          m_nCurrentScene := m_nCurrentScene mod 2;
          Inc(m_nCurrentScene);

          case m_nCurrentScene of
            1:
            begin
              vecEye:= D3DXVector3(40.0, 0.0, -15.0);
              vecAt:=  D3DXVector3(4.0, 4.0, -15.0);
              m_Camera.SetViewParams(vecEye, vecAt);
            end;

            2:
            begin
              vecEye:= D3DXVector3(0.125, 1.25, 3.0);
              vecAt:=  D3DXVector3(0.125, 1.25, 4.0);
              m_Camera.SetViewParams(vecEye, vecAt);
            end;
          end;
        end;

        IDM_TOGGLEHELP: m_bShowHelp := not m_bShowHelp;
      end;
    end;
  end;

  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

end.

